//
// Copyright (c) 2002
// Ronald Kevin Burton
//
// Z poniszym kodem nie jest zwizana adna gwarancja poprawnoci dziaania.
// Program zosta doczony do ksiki ".NET CLR. Ksiga eksperta" w celu
// ilustracji koncepcji i zasad przedstawionych w tej ksice. Program moe by 
// uywany na wasne ryzyko.
//
// Przyznaje si prawo do uycia lub kopiowania tego oprogramowania do dowolnego celu
// bez koniecznoci ponoszenia adnych opat pod warunkiem, e powysze uwagi zostan 
// zachowane we wszystkich kopiach. Przyznaje si take prawo do modyfikacji kodu
// i dystrybucji zmodyfikowanego kodu pod warunkiem zachowania powyszych uwag
// oraz doczenia informacji mwicej o modyfikacji kodu.
//
// 
// Security\SuppressSecurity.cs
using System;
using System.Security;
using System.Security.Permissions;
using System.Runtime.InteropServices;

class NativeMethods
{
    // Wywoanie do niezarzdzanego kodu. Wykonanie tej metody wymaga
    // uprawnienia zabezpiecze UnmanagedCode. Prba wywoania tej metody
    // bez takiego uprawnienia spowoduje zgoszenie wyjatku SecurityException:
    /* UWAGA: Atrybut SuppressUnmanagedCodeSecurity wycza kontrol
       uprawnienia UnmanagedCode w czasie pracy programu. Naley zachowa ostrono! */
    [SuppressUnmanagedCodeSecurity()]
    [DllImport("user32.dll")]
    internal static extern int MessageBox(uint hWnd, string lpText, 
       string lpCaption, uint uType);
}

class MainClass
{
    // Uprawnienie zabezpiecze powizane z t metod spowoduje usunicie
    // uprawnienia UnmanagedCode z aktualnego zestawu uprawnie na czas
    // trwania wywoania tej metody:
    [SecurityPermission(SecurityAction.Deny, Flags = 
       SecurityPermissionFlag.UnmanagedCode)]
    private static void CallUnmanagedCodeWithoutPermission()
    {
        try
        {
            // Kontrola zabezpieczenia UnmanagedCode zostaa wyczona dla poniszego
            // wywoania. Niezarzdzane wywoanie wywietla jedynie interfejs uytkownika.
            // Bezpieczestwo zostanie zachowane tylko wtedy, jeli niezarzdzane
            // wywoanie bdzie posiadao uprawnienie UI:
            UIPermission uiPermission = 
               new UIPermission(PermissionState.Unrestricted);
            uiPermission.Demand();

            Console.WriteLine("Prba wywoania niezarzdzanego kodu bez uprawnienia UnmanagedCode.");
            NativeMethods.MessageBox(0, "Witaj wiecie!", "", 0);
            Console.WriteLine("Wywoano niezarzdzany kod bez uprawnienia UnmanagedCode.");
        }
        catch (SecurityException)
        {
            Console.WriteLine("Przechwycono wyjtek zabezpiecze w czasie prby wywoania niezarzdzanego kodu.");
        }
    }

    // Uprawnienie zabezpiecze powizane z t metod spowoduje dodanie
    // uprawnienia UnmanagedCode do aktualnego zestawu uprawnie na czas
    // trwania wywoania tej metody:
    [SecurityPermission(SecurityAction.Assert, Flags = 
       SecurityPermissionFlag.UnmanagedCode)]
    private static void CallUnmanagedCodeWithPermission()
    {
        try
        {
            Console.WriteLine("Prba wywoania niezarzdzanego kodu z uprawnieniem.");
            NativeMethods.MessageBox(0, "Witaj wiecie!", "", 0);
            Console.WriteLine("Wywoano niezarzdzany kod z uprawnieniem.");
        }
        catch (SecurityException)
        {
            Console.WriteLine("Przechwycono wyjtek zabezpiecze w czasie prby wywoania niezarzdzanego kodu.");
        }
    }

    public static void Main() 
    {
        CallUnmanagedCodeWithoutPermission();
        CallUnmanagedCodeWithPermission();
    }
}
